@@ -146,7 +146,36 @@ class CommonBuildDirectory():
             entry = self.__dir["entries"][gien]
             if entry[CommonBuildDirectory.ENTRY_IDX_NAME] == buildName:
                 entries.append(entry)
-        return entries      
+        return entries  
+        
+    def firstBuildInstanceByName(self, buildName):
+        """
+        Nice JSON of Build - entry 0
+        """    
+        entries = self.lookupBuildEntriesByName(buildName)
+        if len(entries) == 0:
+            return None
+        entry = entries[0]
+        entryInfo = {
+            "label": entry[CommonBuildDirectory.ENTRY_IDX_NAME],
+        }
+        for fi in [
+            (CommonBuildDirectory.ENTRY_IDX_DATE_DISTRIBUTED, "dateDistributed"),
+            (CommonBuildDirectory.ENTRY_IDX_PACKAGE, "package"),
+            (CommonBuildDirectory.ENTRY_IDX_DESCRIPTION, "description"),
+            (CommonBuildDirectory.ENTRY_IDX_FILES, "files"),
+            (CommonBuildDirectory.ENTRY_IDX_RPCS, "rpcs"),
+            (CommonBuildDirectory.ENTRY_IDX_OPTIONS, "options"),
+            (CommonBuildDirectory.ENTRY_IDX_PROTOCOLS, "protocols"),            
+            (CommonBuildDirectory.ENTRY_IDX_HL7LLS, "hl7lls"),            
+            (CommonBuildDirectory.ENTRY_IDX_ROUTINES, "routines"),            
+            (CommonBuildDirectory.ENTRY_IDX_PARAMETER_DEFINITIONS, "parameter_definitions"),            
+            (CommonBuildDirectory.ENTRY_IDX_HL7APS, "hl7_apps")
+        ]:
+            if (len(entry) - 1) < fi[0] or entry[fi[0]] == "":
+                continue
+            entryInfo[fi[1]] = entry[fi[0]]
+        return entryInfo
                
     """
     Plan to support report with: top like file rep, time like user
