@@ -13,6 +13,7 @@ import statistics
 from fmqlutils import VISTA_DATA_BASE_DIR
 from fmqlutils.cacher.cacherUtils import SCHEMA_LOCN_TEMPL 
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
+from fmqlreports.webReportUtils import vistasOfVISNByOne
 
 """
 # TODO: move fmqr/directory
@@ -43,14 +44,6 @@ class CommonFileDirectory():
     # - count pointers, literals etc
     # - boolean on pointers to PATIENT etc
     ENTRY_IDX_FIELDCOUNT = 5
-    
-    # BUILD/INSTALL
-    ENTRY_IDX_BUILD_FIRST_DT = 6
-    ENTRY_IDX_BUILD_LAST_DT = 7
-    ENTRY_IDX_BUILD_COUNT = 8
-    ENTRY_IDX_INSTALL_FIRST_DT = 9
-    ENTRY_IDX_INSTALL_LAST_DT = 10
-    ENTRY_IDX_INSTALL_COUNT = 11
 
     def __init__(self, vistaSNOs):
         self.__vistaSNOs = vistaSNOs
@@ -224,6 +217,31 @@ class CommonFileDirectory():
                 
         return mu
         
+    """
+    Want Summary Info to drive reports
+    """
+    def summaryData(self): 
+        
+        info = {
+    
+            "all": self.fileIds(),
+            "core": self.commonFiles(), # are in all
+            "exclusive": self.commonFiles(count=1), # only in one
+            "stationSpecific": self.stationSpecificFiles(),
+        
+            "singletons": self.filesOfCount(count=1), # if appear, always one
+            "empty": self.filesOfCount(count=0), # if appears, always 0
+            "invalidCounts": self.untrackedFiles(), # files w/o valid counts
+        
+            "totalRecords": self.totalRecords(),
+            "medianRecordsPerFile": self.medianRecords(),
+        
+            "vistas": self.vistas()
+        
+        }
+        
+        return info
+        
     # ----------------- File Ids and Presence across VistAs -------------------
             
     def fileIds(self, stationNo=""):
@@ -495,14 +513,12 @@ string.isalpha() Returns True if there are only alphabetic characters in the str
         )
         
 # ############################## DRIVER ##########################
-        
-VISN20_VISTAS = {"668": "SPO", "663": "PUG", "648": "POR", "687": "WWW", "531": "BOI", "692": "WCO", "653": "ROS"}
-        
+                
 def main():
 
     assert sys.version_info >= (3, 6)
     
-    cfd = CommonFileDirectory(VISN20_VISTAS)
+    cfd = CommonFileDirectory(vistasOfVISNByOne("20"))
     print(cfd.report())
             
 if __name__ == "__main__":
