@@ -267,7 +267,8 @@ class CommonPatientDirectory():
                 cntMFICNs += 1
                 if len(pf[1]) == 1 or len(set(name.split(",")[0] for name in pf[1])) == 1:
                     cntSameName += 1
-                if len(pf[2]) == 2:
+                print(pf)
+                if pf[2] and len(pf[2]) == 2:
                     cntMultiSexes += 1
         mu += "Total: {:,}\n".format(cntMFICNs)
         mu += "W One Name or Same Last Name (good indication that right): {:,}\n".format(cntSameName)
