@@ -570,7 +570,7 @@ class CommonUserDirectory():
             try:
                 usrs = reframeReportUsers(stationNo)
             except:
-                print("Can't load User Reduction for {}".format(stationNo))
+                print("Can't load User Reduction for {} as not available/creatable here".format(stationNo))
                 continue
             changed = True
             print("Loaded USRs for {}: {:,}".format(stationNo, len(usrs)))
