@@ -11,21 +11,23 @@ import io
 import json
 from collections import defaultdict, Counter
 from datetime import datetime
+from dateutil.relativedelta import relativedelta
 
 import pandas as pd
 
 from fmqlutils import VISTA_DATA_BASE_DIR
 from fmqlutils.cacher.cacherUtils import SCHEMA_LOCN_TEMPL 
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
-from fmqlreports.webReportUtils import SITE_DIR_TEMPL, TOP_MD_TEMPL, keyStats, roundFloat
+from fmqlreports.webReportUtils import SITE_DIR_TEMPL, TOP_MD_TEMPL, keyStats, roundFloat, vistasOfVISN
 
-from ifc import muPlotRef, makePlots # TMP (go central to visualizeUtils)
-from webReportFileDirectory import VISN20_VISTAS # TMP (go central to webReportUtils)
+from fmqlreports.webReportUtils import muPlotRef, makePlots 
+
+from webReportDirectoryUtils import webReportSourceVistAs, vistaStationNosToNames, dateRangeOfClones
 from userDirectory import CommonUserDirectory
         
 """
 QUICK TODO: 
-- reorder the render of user totals in time series (last graph) so biggest appears first ie/ key has PUG, POR ... vistaNamesPlus needs to be ordered
+- reorder the render of user totals in time series (last graph) so biggest appears first ie/ key has PUG, POR ... colNameById needs to be ordered
 
 Goal for directories in general - user ala patient with one, move id checks there but
 first need to see what's there now. Federated Directory => mismatch possible, dup,
@@ -130,7 +132,7 @@ def makeTotalsSection(cd, vistaNames, imageDir="Images"):
             "plotMethod": "plotCategoryBSV",
             "specs": commonTotals
     }
-    blurb = "The graph below shows the total number of users in each system as well as a breakdown of whether those users are unique to that system (exclusive), in some but not all systems (some) or in every (all) other systems in the directory. It's striking how few of the users in the smaller systems - Walla Walla (WWW) and Spokane (SPO) - are unique to them. Indeed, for WWW, most of its users appear in some or every other system."
+    blurb = "The graph below shows the total number of users in each system as well as a breakdown of whether those users are unique to that system (exclusive), in some but not all systems (some) or in every (all) other systems in the directory. It's striking how few of the users in the smaller systems - for example, Walla Walla (WWW) - are unique to them. Indeed, for WWW, most of its users appear in some or every other system."
     mu += "{}\n\n{}\n\n".format(
         blurb,
         muPlotRef(plotData["commonByVistA"], imageDir)
@@ -165,25 +167,31 @@ Stories shown User:
 
 def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
 
-    plotData = {}
-    
-    end = "2020-02" # so common for all (make dynamic going forward)
+    dtRangeClones = dateRangeOfClones(vistaNames) 
+    oldestLastFullMonth = dtRangeClones["oldestLastFullMonth"]
 
+    plotData = {}
+    end = oldestLastFullMonth.strftime("%Y-%m")
+    startDT = oldestLastFullMonth - relativedelta(months=1)
+    start = startDT.strftime("%Y-%m")
+    startTwoYearsBack = (startDT - relativedelta(years=2)).strftime("%Y")
+    startFiveYearsBack = (startDT - relativedelta(years=5)).strftime("%Y")
+    
     mu = "## User Additions over Time\n\n"
     
     dailiesDFCSV = csvDatesBySNO(cd)
         
-    vistaNamesPlus = vistaNames.copy()
-    vistaNamesPlus["USER"] = "New" # new users
+    colNameById = vistaNames.copy()
+    colNameById["USER"] = "New" # new users
     
     plotData["tsDailies"] = {
-        "title": "2020/01-{} User Addition".format(re.sub(r'\-', '/', end)),
+        "title": "{}-{} User Addition".format(re.sub(r'\-', '/', start), re.sub(r'\-', '/', end)),
         "plotName": f'{PLOT_PREFIX}',
         "plotMethod": "plotDailies",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "vistaNamesPlus": vistaNamesPlus,
-            "start": "2020-01",
+            "colNameById": colNameById,
+            "start": start,
             "end": end
         }
     }
@@ -202,12 +210,12 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
         "plotMethod": "plot365RollingMean",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "vistaNamesPlus": vistaNamesPlus,
+            "colNameById": colNameById,
             "start": "1987-01", 
             "end": end
         }
     }
-    blurb = "Here are the yearly trends using a __\"365 day rolling mean\"__ starting when the directory first gets data to the latest additions. In general, additions follow system size (Puget, Portland, Spokane, Walla Walla) but there's an exception in 2019 where four of the seven have a large spike in additions and Portland exceeds Puget's total. This is one of four times where the rate of addition changes - the others are 2002, 2006 and 2009. _All need examination_ - they must reflect the addition of or changes to VistA applications and use."
+    blurb = "Here are the yearly trends using a __\"365 day rolling mean\"__ starting when the directory first gets data to the latest additions. In general, additions follow system size but there's an exception in 2019 where four of the seven have a large spike in additions and Portland exceeds Puget's total. This is one of four times where the rate of addition changes - the others are 2002, 2006 and 2009. _All need examination_ - they must reflect the addition of or changes to VistA applications and use."
     mu += "{}\n\n{}\n\n".format(
         blurb,
         muPlotRef(
@@ -217,17 +225,17 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
     )
     
     plotData["tsMonthlyResamples"] = {
-        "title": "Users - Resampled Monthly (2018 - {})".format(re.sub(r'\-', '/', end)),
+        "title": "Users - Resampled Monthly ({} - {})".format(startTwoYearsBack, re.sub(r'\-', '/', end)),
         "plotName": f'{PLOT_PREFIX}',
         "plotMethod": "plotMonthlyResamples",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "vistaNamesPlus": vistaNamesPlus,
-            "start": '2018-01', 
+            "colNameById": colNameById,
+            "start": f'{startTwoYearsBack}-01', 
             "end": end
         }
     }
-    blurb = "When the data is __resampled by month__, it shows three subpeaks for the 2019 leap in additions. The reason for Portland's leap above Puget also shows more clearly - though Puget has one large spike in April 2019, Portland has two, one in July and one in September. Note that the lack of any spike mirrors the flat line for smaller systems during this period in the yearly trends above.\n\nThis graph also shows - in purple - the running total of new users added. As seen above, users added to VistAs sometimes already pre-exist in other VistAs. While the new user line shadows Puget's peak in April and Portland's in September, it is markedly lower than the spikes for Portland and Spokane in July - this indicates that much of those spikes are existing (Puget?) users being added to these systems. _This needs examination_."
+    blurb = "When the data is __resampled by month__, it shows three subpeaks for the 2019 leap in additions. The reason for Portland's leap above Puget also shows more clearly - though Puget has one large spike in April 2019, Portland has two, one in July and one in September. Note that the lack of any spike mirrors the flat line for smaller systems during this period in the yearly trends above.\n\nThis graph also shows (\"New\") the running total of new users added. As seen above, users added to VistAs sometimes already pre-exist in other VistAs. While the new user line shadows Puget's peak in April and Portland's in September, it is markedly lower than the spikes for Portland and Spokane in July - this indicates that much of those spikes are existing (Puget?) users being added to these systems. _This needs examination_."
     mu += "{}\n\n{}\n\n".format(
         blurb,
         muPlotRef(
@@ -237,13 +245,13 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
     )
   
     plotData["tsTotalsPerYear"] = {
-        "title": "Users - Annual Additions (2015 - {})".format(re.sub(r'\-', '/', end)),
+        "title": "Users - Annual Additions ({} - {})".format(startFiveYearsBack, re.sub(r'\-', '/', end)),
         "plotName": f'{PLOT_PREFIX}',
         "plotMethod": "plotTotalsPerYear",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "vistaNamesPlus": vistaNamesPlus,
-            "start": "2015", 
+            "colNameById": colNameById,
+            "start": startFiveYearsBack, 
             "end": end
         }
     }
@@ -377,8 +385,9 @@ MORE:
 - https://levelup.gitconnected.com/pandas-dataframe-python-10-useful-tricks-b4beae91df3d ... filtering, combining, renaming in frames ie/ all frame operations to try
 - https://medium.com/@rrfd/cleaning-and-prepping-data-with-python-for-data-science-best-practices-and-helpful-packages-af1edfbe2a3 <---- includes 'fillna'
 """
-def csvDatesBySNO(cd):
-        datesBySNO = cd.entryDatesBySNO()
+def csvDatesBySNO(cd=None, datesBySNO=None):
+        if datesBySNO == None:
+            datesBySNO = cd.entryDatesBySNO()
         snosOrdered = sorted(datesBySNO, key=lambda x: len(datesBySNO[x]), reverse=True)
         byDateBySNO = defaultdict(Counter)
         for sno in datesBySNO:
@@ -407,8 +416,11 @@ def main():
             return
         raise Exception("Only argument allowed is 'PLOT'")
 
+    visn20VistAs = vistasOfVISN("20")
     dirName = "User"
-    cud = CommonUserDirectory(VISN20_VISTAS)
+    cud = CommonUserDirectory(visn20VistAs)
+    snosInDir = set(entry["stationNumber"] for entry in cud.vistas())
+    visn20VistAsSupported = dict((sno, visn20VistAs[sno]) for sno in visn20VistAs if sno in snosInDir)
 
     plotData = {}
     mu = f"""---
@@ -420,22 +432,26 @@ title: VISN 20 {dirName} Directory
 
 """
 
-    mu += """The following reports on a VISN 20 User Directory built from the user file (200) of 7 VistAs: Puget Sound [663], Portland [648], Boise [531], Spokane [668], Roseburg [653], White City [692] and Walla Walla [687]. Together they hold <span class='yellowIt'>{:,}</span> distinct users using <span class='yellowIt'>{:,}</span> user file records.
-
-Note: Anchorage is the missing VISN 20 system. Its information will be added once the system is made available.
+    snosToNames = vistaStationNosToNames(visn20VistAsSupported)
+    vistasInDirMU = ", ".join([f'{snosToNames[sno]} [{sno}]' for sno in snosToNames])
+    mu += """The following reports on a VISN 20 User Directory built from the user file (200) of <span class='yellowIt'>{:,}</span> VistAs: {}. Together they hold <span class='yellowIt'>{:,}</span> distinct users using <span class='yellowIt'>{:,}</span> user file records.
 
 """.format(
+        len(visn20VistAsSupported),
+        vistasInDirMU, 
         cud.total(),
         cud.entryTotal()
     )
     
-    ttlPlotData, ttlMU = makeTotalsSection(cud, VISN20_VISTAS)
+    ttlPlotData, ttlMU = makeTotalsSection(cud, visn20VistAsSupported)
     mu += ttlMU
     plotData.update(ttlPlotData)
     
-    tsPlotData, tsMU = makeTimeSeriesSection(cud, VISN20_VISTAS)
+    tsPlotData, tsMU = makeTimeSeriesSection(cud, visn20VistAsSupported)
     mu += tsMU
     plotData.update(tsPlotData)
+    
+    mu += webReportSourceVistAs(visn20VistAsSupported)
         
     plotDir = f'{VISTA_DATA_BASE_DIR}Common/TmpWorking/'
     print(f'Serializing vizData to {plotDir}')
@@ -447,4 +463,4 @@ Note: Anchorage is the missing VISN 20 system. Its information will be added onc
         
 if __name__ == "__main__":
     main()
-    
\ No newline at end of file
+    
