@@ -245,7 +245,11 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
             placerPatients |= servicePatients
             placerTTL += serviceTTL
             if len(set(rf for st in byServiceWRF[service] for rf in st["routing_facility"]["byValueCount"])) > 1:
-                raise Exception("Expected one and only one RF for Placer Services")
+                # stricter to reflect Alaska where DISCONTINUED 
+                # only st with a different RF
+                if len(set(rf for st in [st for st in byServiceWRF[service] if sum(1 for status in st["cprs_status"]["byValueCount"] if not re.match(r'DISCONTINUE', status))] for rf in st["routing_facility"]["byValueCount"])) > 1:
+                    raise Exception(f"Expected one and only one RF for Placer Services unless all entries of other are discontinued") # happened in Alaska 
+                # TODO: consider nixing this (and PENDING?) all together
                 
     if (placerTTL + fillerTTL + worfTTL) != allTTL:
         raise Exception("Totalling mistake - consult totals are exclusive")
@@ -453,13 +457,18 @@ utomatically generated note - signature not required.\n \n  Electronically Filed
  ",
     """
     def enforceOnlyTelerHasARnDoc(st):
+        if len(st["cprs_status"]["byValueCount"]) == 1 and re.match(r'PENDING', singleValue(st, "cprs_status")):
+            return # for Alaska, allow pending (may be rogue) through TODO
         if "associated results" in st:
-            raise Exception("WRF: TELER is only one expected to have local associated results")
+            print(f"WRF: TELER is only one expected to have local associated results")
+        # TODO 653: 6/21 has Mental Health Note
         if "tiu_result_narrative" in st:
-            raise Exception("WRF: TELER is only one expected to have local tiu - equivalent to associated resutls")
+            print(f"** WRF: TELER is only one expected to have local tiu - equivalent to associated resutls - {st['tiu_result_narrative']['byValueCount']}")
     for i, service in enumerate(sorted(placerServices, key=lambda x: sum(st["_total"] for st in byServiceWRF[x]), reverse=True), 1):
         tttl = sum(st["_total"] for st in byServiceWRF[service])
-        rf = singleValue(byServiceWRF[service][0], "routing_facility")
+        # Assuming one ok - consistency ensured/enforced above: allowed > 1 if
+        # all DISCONTINUE (Alaska). Taking rf of the one with the most entries.
+        rf = singleValue(sorted(byServiceWRF[service], key=lambda x: x["_total"], reverse=True)[0], "routing_facility")
         placerServicesFacilities[rf] += tttl
         if service in telerAcServicesUsed:
             groupLabel = "TELER"
