@@ -14,28 +14,57 @@ from fmqlutils.cacher.cacherUtils import FMQLReplyStore, FilteredResultIterator,
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent, muBVC
 from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVCOfSTProp
 
-from ..webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce200, reduce4, flattenPropValues
+from fmqlreports.webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce200, reduce4, flattenPropValues
+
+# CPRS RELEASED ORDER/RECEIVED/COMPLETE-UPDATE/NEW NOTE ADDED (no NOTE ADDED in boi)
+# CPRS RELEASED ORDER/COMPLETE-UPDATE/NEW NOTE ADDED
+# CPRS RELEASED ORDER/COMPLETE-UPDATE
+# CPRS RELEASED ORDER/RECEIVED/INCOMPLETE RPT/COMPLETE-UPDATE/NEW NOTE ADDED
 
 """
-TODO: ac finish
-- 4 way service including enforce for routing
-  ... and need reduce4 so using stationNo
-  
-REDO ACQ ACTIVITY AGAIN TO:
-- break by READING VistA ie/ same rep but per reading VistA!
-- graphics of activity ... see if redo WWW ... per week/day/month? and per reader?
-  ... over the year / per activity type (ie/ per service?)
-  ... another go ala s... diagram
-------
-- idea (if available) of patient reg delay
-THEN DO READER SIDE AND THEIR CONSULTs ... needs breakdown ---- MAY BE FRIDAY
-"""
+TODO 1: Markup
+- local ac config broken
+- local resolution too
+- http://localhost:9100/schema#8925_91 ... external data link and images in FILER side? for TELER?
+
+TODO 2: Grabber Viola (668, 692, 531 (seattle does some DERM), 653, 687)
+-------
+before READER side: 'grabber' VIOLA (HOUSTON, VIOLA D -- not POR but others)
+- st is all canceled? (is in ROSEBURG)
+- need to isolate the grabbers and show those in isolation vs the readers (only lucked out on Viola)
+- are viola's signons long for TELER ie/ TELER session duration
+- in 663 report, VIOLA is someone setup for READING ... need her title
+- do see grabber TeleR usage
+- on Reader/Puget side, does Viola appear in Consults as an allocator?
+
+> rem I said Puget (which does nearly all DERM => most TeleR as most TeleR is DERM) doesn't use TELER GUI? And we need to stay clear of this lack of use? Well that's true iff use == lock. It looks like they have one or more clerks who DO use TELER GUI to see work list but than rather than locking, they allocate consults to doctors using some other method. We'll bring out such "work allocators" in our CLIN 3 demo for the week after next.
+
+TODO 2: Report misleading for dominant reading VistAs including from themselves (queue probably bypassed and just do local work) ie/ LOCALs blend.
+--------
+        - highlight active services up front
+        - locals distinguished ... ie/ not just last table in list (+ don't bold names)
+        - ACTIVITY: % of LOCAL vs REMOTE
+------- <-------- see local ac service table near top and how plays out
+- POR 648 and all its DENTISTRY!!! BUT ARE THEY ALL LOCAL? NEED BETTER LOCAL/REMOTE SPLIT ala READER / ACQ split ... ie/ within ACQ -- read locally? [so not really tele?]
+- see in table in ac for "REMOTE READER" ... not set as local so 663 DERM is local! Even EYE is ... only a few BOISE appear (ELCYZYN, TIMOTHY E -- he has a lot of TELER sessions though ie/ keeps trying?)
+- BOI 531 -- ELCYZYN, TIMOTHY E [97094] reads locally too needs to come out ie/ uses it locally ... see table on top of 4 LOCAL services
+  (contrast with WCO where its locals go unused? -- need to note that up top)
+
+TODO MORE:
+----------
+- reader side report (incoming IFC ie/ filer) ... will tie in with documents <-- big for 663 and 648
+- [2006.5831](http://localhost:9100/schema#2006_5831)
+- may put queue trigger back in if the behavior is significant
+- note if activity (ie/ if active in last year) is used in config section
 
-"""
 Report TeleReading and the supporting IFCs, documents and images. Very particular IFC.
 Cover [1] both Acquisition and Reader sides (including when they are the same VistA) and
 [2] the configuration (files) used in all cases.
 
+ADD BACK NOTE
+
+The client __[Telereader Configurator](https://www.va.gov/vdl/documents/Clinical/Vista_Imaging_Sys/mag_telereader_configuration.pdf)__ sets up the following configuration files that determine how Telereading operates in a VistA. It configures both the __Acquisition Role__ and the __Reader Role__.
+
 BETTER INTERP/CONFIG SETUPs (not even clear to me!):
 - what's with Boise stuff? 
 - remote service name vs local? ie/ one portland 'remote' "BETIC TELERETINAL IMAGING (PORTL" / 2006.5841 has many local 123.5 service types (where one each per local site like Yakama) 
@@ -99,7 +128,7 @@ def webReportTeleReading(stationNo, stationName, onAndAfterDay, upToDay):
         # REQUEST/CONSULTATION for T. Q YR1E - complication of local-only Telereading
         {"fileType": "123", "check": "YR1"}, 
         # REQUEST SERVICES (local terms mapped to VA-wide in 2006_5841)
-        {"fileType": "123_5", "check": "ALL"},
+        {"fileType": "123_5", "check": "ALL", "stoppedToo": True},
 
         # For User Identification
         # 200 - reduce200 will enforce need for 200
@@ -107,29 +136,131 @@ def webReportTeleReading(stationNo, stationName, onAndAfterDay, upToDay):
         
     ])
     if not allThere:
-        raise Exception("Some required data is missing - {}".format(details))
+        raise Exception(f"Some required data is missing: {json.dumps(details, indent=4)}")
 
     mu = TOP_MD_TEMPL.format("{} TeleReader".format(stationNo))
 
     mu += """# Telereader
    
-The following describes _Telereading_ in VistA _{}_ [{}]. There are three scenarios [i] _Acquisition Role_ (Acquisition Local, Reader Remote), [ii] _Reader Role_ (Acquisition Remote, Reader Local), [iii] _Acquisition and Reader Role_ (Acquisition Local, Reader Local).
+The following describes _Telereading_ in VistA _{}_ [{}] cut on {}. VistA can both acquire studies (\"Acquisition\") and read studies (\"Reading\"). Where one VistA acquires a study and another reads that study, Telereading builds upon VistA's general-purpose Inter-facility Consult (IFC) feature.
 
     
-""".format(stationName, stationNo)
+""".format(stationName, stationNo, upToDay) 
 
-    mu += webReportConfiguration(stationNo)
-
-    mu += webReportAcquisition(stationNo, onAndAfterDay, upToDay)
+    mu += webReportAcquisition(stationNo, stationName, onAndAfterDay, upToDay)
+    
+    mu += webReportReading(stationNo, stationName, onAndAfterDay, upToDay)
     
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     print("Serializing Report to {}".format(userSiteDir))
-    open(userSiteDir + "telereader2.md", "w").write(mu)
+    open(userSiteDir + "telereader.md", "w").write(mu)
     
-# ######################## Acquisition Activity ######################
+# ######################## Acquisition ######################
+    
+def webReportAcquisition(stationNo, stationName, onAndAfterDay, upToDay):
+    mu = "## Acquisition\n\n" 
+    mu += webReportAcquisitionConfiguration(stationNo, stationName)
+    mu += webReportAcquisitionActivity(stationNo, onAndAfterDay, upToDay)
+    return mu
     
 """
-Acquisition - local acquisition and either remote or local telereading
+Acquisition Service is effectively a combination of information from 123.5 and 2006_5841
+
+TODO:
+- consider a synthesized name of SPEC PROC ABBREVS (SNO AC/SNO READ VISTA)
+"""
+def webReportAcquisitionConfiguration(stationNo, stationName):
+
+    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
+    ifcServiceIndex = dict((cs["_id"], cs) for cs in FilteredResultIterator(dataLocn, "123_5") if "ifc_routing_site" in cs)
+    institByIEN = reduce4(stationNo)
+    snoByNationalInstitId = dict(("4-" + ien, institByIEN[ien]["station_number"]) for ien in institByIEN if "isNational" in institByIEN[ien] and "station_number" in institByIEN[ien])
+    
+    asResourceIter = FilteredResultIterator(dataLocn, "2006_5841")
+    serviceCount = 0
+    byRoutingSite = defaultdict(list)
+    locals = []
+    for resource in asResourceIter:
+        serviceCount += 1
+        row = [
+            "__{}__".format(re.sub(r' +$', '', re.sub(r'^ +', '', resource["name"]["label"]))),
+            "{} [{}]".format( # 2006.5841
+                re.sub(r'( VAMC| VA CLINIC| CBOC| VA MEDICAL CENTER)', '', resource["acquisition_site"]["label"]),
+                snoByNationalInstitId[resource["acquisition_site"]["id"]]
+            ),
+            "{} [{}]".format( # 2006.5841
+                resource["specialty_index"]["label"],
+                resource["specialty_index"]["id"].split("-")[1]
+            ),
+            "{} [{}]".format( # 2006.5841
+                resource["procedure_index"]["label"],
+                resource["procedure_index"]["id"].split("-")[1]
+            )
+        ]
+        tiuNoteFile = resource["tiu_note_file"]["label"] if "tiu_note_file" in resource else ""
+        row.append(tiuNoteFile)
+        # If not just LOCAL
+        if resource["name"]["id"] in ifcServiceIndex: # 123.5
+            ifcService = ifcServiceIndex[resource["name"]["id"]]
+            row.append(ifcService["ifc_remote_name"])
+            routingSite = ifcService["ifc_routing_site"]
+            remoteSiteRef = "{} [{}]".format(
+                routingSite["label"],
+                snoByNationalInstitId[routingSite["id"]]
+            )
+            byRoutingSite[remoteSiteRef].append(row)
+            # NOT DOING ASSOCIATED STOP CODE as many don't have it
+        else:
+            locals.append(row)
+        # IGNORING FOR NOW (as doesn't effect quantifying per se
+        # trigger = resource["unread_list_creation_trigger"]
+
+    mu = "### Configuration\n\n"
+    mu += "This VistA has <span class='countHigh'>{:,}</span> consult services for Telereading by <span class='countHigh'>{:,}</span> remote VistAs.{}\n\n".format(
+        serviceCount,
+        len(byRoutingSite),
+        " In addition, <span class='countHigh'>{:,}</span> services are for local reading.".format(len(locals)) if len(locals) else ""
+    )
+    basicCols = [
+            ":Service",
+            ":Acquisition Site",
+            ":Specialty",
+            ":Procedure",
+            ":Acquisition Note"
+    ]
+    for rs in sorted(byRoutingSite):
+        # "All map to one service in Boise
+        acSites = set(row[1] for row in byRoutingSite[rs])
+        remoteServices = set(row[-1] for row in byRoutingSite[rs])
+        cols = basicCols[:]
+        if len(remoteServices) > 1:
+            cols.append(":Remote Service")
+        tbl = MarkdownTable(cols)
+        for row in sorted(byRoutingSite[rs], key=lambda x: x[1]): # group by ac site
+            if len(remoteServices) == 1:
+                row.pop()
+            tbl.addRow(row)
+        if len(acSites) > 1:
+            mu += "__{}__ receives <span class='countHigh'>{:,}</span> types of consult service from <span class='countHigh'>{:,}</span> acquisition sites{} ...\n\n".format(
+                rs, 
+                len(byRoutingSite[rs]), 
+                len(acSites),
+                ", all sent to one remote service _{}_".format(list(remoteServices)[0]) if len(remoteServices) == 1 else "" 
+            )
+        else:
+            mu += "__{}__ receives <span class='countHigh'>{:,}</span> services ...\n\n".format(rs, len(byRoutingSite[rs]))
+        mu += tbl.md() + "\n\n"
+    if len(locals):
+        tbl = MarkdownTable(basicCols)
+        mu += "The <span class='countHigh'>{:,}</span> locally read (non interfacility consult) services are ...\n\n".format(len(locals))
+        for row in sorted(locals, key=lambda x: x[0]):
+            tbl.addRow(row)
+        mu += tbl.md() + "\n\n"
+    
+    return mu
+
+"""
+Acquisition Activity - local acquisition and either remote or local telereading
 
 Different breaks of Enhanced 2006_5849 (with a little addition from 3.081's
 app breakdown)
@@ -157,7 +288,7 @@ e. for service: tiu note type (ie/ one or none)
 
 number_of_images -- print it
 """
-def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
+def webReportAcquisitionActivity(stationNo, onAndAfterDay, upToDay):
 
     # 123 and 2006_5849 reductions
     type123YR1, sts123YR1 = splitTypeDatas(stationNo, "123", reductionLabel="YR1", )
@@ -182,12 +313,12 @@ def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
         print("Loading user information (200 Reduction) ...")
         userInfoByUserRef = reduce200(stationNo, coreOnly=True)   
         # V1.1 and V10 compatible as creator only shows if != 200-0 for v1.1
-        readersAutoCreated = set(userRef for userRef in userInfoByUserRef if userRef in readers and "creator" not in userInfoByUserRef[userRef])
+        readersAutoCreated = dict((userRef, [userInfoByUserRef[userRef]["date_entered"], userInfoByUserRef[userRef]["hasProps"]]) for userRef in userInfoByUserRef if userRef in readers and "creator" not in userInfoByUserRef[userRef])
         print("\tLoaded and processed {:,} users, {:,} readers to see {:,} are auto created".format(len(userInfoByUserRef), len(readers), len(readersAutoCreated)))
         return readersAutoCreated
     readersAutoCreated = autoCreatedReaders(readersPerAcVistA)
                             
-    mu = "## Telereader Acquisition\n\n"             
+    mu = "### Activity\n\n"             
 
     """
     Overall summary uses:
@@ -203,7 +334,20 @@ def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
         ) if "ifc_role" in type2006_5849YR1E else 0,
     """
     consultIFCCount = type2006_5849YR1E["is_consult_ifc"]["count"] if "is_consult_ifc" in type2006_5849YR1E else 0
-    mu += """An _Acquisition site_ imports images and submits them for telereading \"consults\". These are tracked in _TELEREADER READ/UNREAD LIST (2006.5849)_. In this site, there are <span class='countHigh'>{:,}</span> total consults between {} and {}. Of these, <span class='countHigh'>{}</span> are in this LIST, <span class='countHigh'>{}</span> of these are interfacility consults (IFCs), where another facility performs the consult while the balance, <span class='countHigh'>{}</span>, are performed locally.
+    if consultIFCCount < type2006_5849YR1E["_total"]:
+        mixedOrNotMU = "<span class='countHigh'>{}</span> of these are interfacility consults (IFCs), where another facility performs the consult while the balance, <span class='countHigh'>{}</span>, are performed locally".format(
+            reportAbsAndPercent(
+                consultIFCCount, 
+                type2006_5849YR1E["_total"]
+            ),
+            reportAbsAndPercent(
+                type2006_5849YR1E["_total"] - consultIFCCount, 
+                type2006_5849YR1E["_total"]
+            )
+        )
+    else:
+        mixedOrNotMU = "all of which are interfacility consults (IFCs) - no locally acquired studies are read locally"
+    mu += """An _Acquisition site_ imports images and submits them for telereading \"consults\". These are tracked in _TELEREADER READ/UNREAD LIST (2006.5849) [\"Acquisition List\"]_. In this site, there are <span class='countHigh'>{:,}</span> total consults between {} and {}. Of these, <span class='countHigh'>{}</span> are in the Acquisition List, {}.
     
 """.format(
         type123YR1["_total"],
@@ -211,16 +355,7 @@ def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
         upToDay,
         # count of queue consults == count of queue entries as 1-1
         reportAbsAndPercent(type2006_5849YR1E["_total"], type123YR1["_total"]), 
-        # E has ifc_role from Consult (always PLACER) and it's this or its local
-        reportAbsAndPercent(
-            consultIFCCount, 
-            type2006_5849YR1E["_total"]
-        ) if consultIFCCount else 0,
-        # if not IFC, then local
-        reportAbsAndPercent(
-            type2006_5849YR1E["_total"] - consultIFCCount, 
-            type2006_5849YR1E["_total"]
-        ) if consultIFCCount < type2006_5849YR1E["_total"] else 0
+        mixedOrNotMU
     ) 
 
     """
@@ -244,22 +379,17 @@ def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
         stsBySpecialty[specialty].append(st)
     for specialty in sorted(stsBySpecialty, key=lambda x: sum(st["_total"] for st in stsBySpecialty[x]), reverse=True):
         mu += "Specialty _{}_ ...\n\n".format(re.split(r' +\[', specialty)[0])
-        tbl = MarkdownTable([":Service", ":Remote Reader", "Count", ":Status", ":TIU Note Type"])
+        tbl = MarkdownTable([":Service", ":Remote Reader", "Count", ":Status"])
         for stService in sorted(stsBySpecialty[specialty], key=lambda x: x["_total"], reverse=True):
             service = singleValue(stService, "consult_service")
             serviceIEN = re.search(r'\-(\d+)', service).group(1)
             name = re.split(r' +\[', service)[0]
-            ifcRoutingSiteMU = re.sub(r'4\-', '', ifcServiceByIEN[serviceIEN]["ifc_routing_site"]) if serviceIEN in ifcServiceByIEN and "ifc_routing_site" in ifcServiceByIEN[serviceIEN] else ""
-            if "consult_service_tiu_note_type" in stService:
-                tiu_note_type = singleValue(stService, "consult_service_tiu_note_type").split(" [")[0]
-            else:
-                tiu_note_type = ""
+            ifcRoutingSiteMU = re.sub(r'4\-', '', ifcServiceByIEN[serviceIEN]["ifc_routing_site"]) if serviceIEN in ifcServiceByIEN and "ifc_routing_site" in ifcServiceByIEN[serviceIEN] else "-"
             tbl.addRow([
-                "__{}__".format(name) if ifcRoutingSiteMU else name,
+                "__{}__".format(name) if ifcRoutingSiteMU != "-" else name,
                 ifcRoutingSiteMU,
                 stService["_total"], 
-                muBVC(stService["status"]["byValueCount"]),
-                tiu_note_type
+                muBVC(stService["status"]["byValueCount"])
             ])
         mu += tbl.md() + "\n\n"
     
@@ -306,13 +436,14 @@ def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
                 tbl.addRow(row)
     mu += """List entries go through one or more transitions, starting with a _CPRS RELEASED ORDER_ when the entry is first added right after its consult is created. The most common sequence moves on to _RECEIVED_ (reader site received the consult) through _COMPLETE UPDATE_ (reader site has read the image and created a document) and ends with _NEW NOTE ADDED_ (an equivalent document was created in the \"local\"/acquisition site). 
     
-The following shows transitions by list entry status. The last column shows how _ADDED COMMENT_ actions accompany a specific transition. These actions may be local or remote and allow users to annotate a consult. Transitions in __bold__ represent remote reader activity ...
+The following shows transitions by Acquisition List entry status. The last column shows how _ADDED COMMENT_ actions accompany a specific transition. These actions may be local or remote and allow users to annotate a consult. Transitions in __bold__ represent remote reader activity ...
 
 """
     mu += tbl.md() + "\n\n"
     
     """
     Per Reader (User) summaries: st2006_5849Us - will distinguish remote and local 
+    ... on the acquisition side ie for the 'Acquisition List'
     
     TODO: ala older report
     - network_name (so portable for common view)
@@ -360,233 +491,43 @@ The following shows transitions by list entry status. The last column shows how
                     remoteMU += " [{}]".format(list(st["reader_duz_at_reading_site"]["byValueCount"])[0])
         if userRef not in readersAutoCreated:
             nameMU = re.sub(r'\]', '*]', nameMU)
+            detailsMU = ""
+        else:
+            detailsMU = f'{readersAutoCreated[userRef][0]}/{len(readersAutoCreated[userRef][1]):,}/{"ssn" in readersAutoCreated[userRef][1]}'
         tbl.addRow([
             nameMU,
+            detailsMU,
             re.sub(r'(ATOLOGY| CARE)', '', muBVC(st["image_index_for_specialty"]["byValueCount"])),
             st["_total"],
             remoteMU,
             appsMU
         ])
-    mu += """For the period considered here, the list has <span class='countHigh'>{:,}</span> Readers, <span class='countHigh'>{}</span> of which are local. The names of Readers from remote sites (_\"Remote Readers\"_) are in __bold__.
+    mu += """For the period considered here, the Acquisition List has <span class='countHigh'>{:,}</span> Readers, <span class='countHigh'>{}</span> of which are local with the balance from remote reading sites. The names of Readers from these remote reading sites (_\"Remote Readers\"_) are in __bold__. <span class='countHigh'>{}</span> of the following Readers were added automatically to this VistA - the minority added explicitly are marked with a _*_. 
     
 """.format(
         len(st2006_5849Us), 
-        reportAbsAndPercent(noLocal, len(st2006_5849Us))
-    )
-    mu += """<span class='countHigh'>{}</span> of the following Readers were added automatically to this VistA - those added explicitly are marked with a _*_. \"Applications (Signons)\" gives the number of sign ons a user makes with different applications for the period under consideration. _UNSET_ means the application doesn't identify itself - _CPRS_ is the chief example of this. Note that _TeleReader_ is used by remote users.
-    
-""".format(
+        reportAbsAndPercent(noLocal, len(st2006_5849Us)),
         reportAbsAndPercent(
             len(readersAutoCreated),
             len(st2006_5849Us)
         )
     )
+    mu += """\"#\" counts queue entries of a user while \"Applications (Signons)\" gives the number of sign ons a user makes with different applications for the period under consideration. _UNSET_ means the application doesn't identify itself - _CPRS_ is the chief example of this. Note that _TeleReader_ is used by remote users.
+    
+"""
     mu += tbl.md() + "\n\n"
         
     return mu  
-    
+        
 # ################## Configuration (Reader and Acquisition Roles) ###############
     
-"""
-As setup by Telereader configuration - decide how reader operates. Note that for events
-from reading, can fill in many extra properties using these configurations.
-"""
-def webReportConfiguration(stationNo):
-
-    mu = """
-    
-## Configuration
-
-The client __[Telereader Configurator](https://www.va.gov/vdl/documents/Clinical/Vista_Imaging_Sys/mag_telereader_configuration.pdf)__ sets up the following configuration files that determine how Telereading operates in a VistA. It configures both the __Acquisition Role__ and the __Reader Role__.
-
-"""
-
-    mu += webReportConfigurationAcquisition(stationNo)
-    mu += webReportConfigurationReader(stationNo)
-    
-    return mu
-
-"""
-Acquisition Service is effectively a combination of information from 123.5 and 2006_5841
-
-TODO:
-- 
-- tie in tiu, creation trigger (once I'm done with it)
-- consider a synthesized name of SPEC PROC ABBREVS (SNO AC/SNO READ VISTA)
-"""
-def webReportConfigurationAcquisition(stationNo):
-
-    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
-    ifcServiceIndex = dict((cs["_id"], cs) for cs in FilteredResultIterator(dataLocn, "123_5") if "ifc_routing_site" in cs)
-    institByIEN = reduce4(stationNo)
-    snoByNationalInstitId = dict(("4-" + ien, institByIEN[ien]["station_number"]) for ien in institByIEN if "isNational" in institByIEN[ien] and "station_number" in institByIEN[ien])
-    
-    asResourceIter = FilteredResultIterator(dataLocn, "2006_5841")
-    serviceCount = 0
-    byRoutingSite = defaultdict(list)
-    locals = []
-    for resource in asResourceIter:
-        serviceCount += 1
-        row = [
-            "__{}__".format(resource["name"]["label"]),
-            "{} [{}]".format( # 2006.5841
-                resource["acquisition_site"]["label"],
-                snoByNationalInstitId[resource["acquisition_site"]["id"]]
-            ),
-            "{} [{}]".format( # 2006.5841
-                resource["specialty_index"]["label"],
-                resource["specialty_index"]["id"].split("-")[1]
-            ),
-            "{} [{}]".format( # 2006.5841
-                resource["procedure_index"]["label"],
-                resource["procedure_index"]["id"].split("-")[1]
-            )
-        ]
-        # If not just LOCAL
-        if resource["name"]["id"] in ifcServiceIndex: # 123.5
-            ifcService = ifcServiceIndex[resource["name"]["id"]]
-            row.append("__{}__".format(ifcService["ifc_remote_name"]))
-            routingSite = ifcService["ifc_routing_site"]
-            remoteSiteRef = "{} [{}]".format(
-                routingSite["label"],
-                snoByNationalInstitId[routingSite["id"]]
-            )
-            byRoutingSite[remoteSiteRef].append(row)
-            print(ifcService["associated_stop_code"])
-        else:
-            locals.append(row)
-        trigger = resource["unread_list_creation_trigger"]
-        tiuNoteFile = resource["tiu_note_file"] if "tiu_note_file" in resource else ""
-        # associatedStopCode = "associated_stop_code
-    # ADD NOTE:
-    # MAND ... ADD STOP CODES as they are using em!!!!!
-    mu = "### Acquisition Role\n\n"
-    mu += "There are <span class='countHigh'>{:,}</span> consult services for Telereading by <span class='countHigh'>{:,}</span> remote sites.{}\n\n".format(
-        serviceCount,
-        len(byRoutingSite),
-        " In addition, <span class='countHigh'>{:,}</span> services are read locally.".format(len(locals)) if len(locals) else ""
-    )
-    for rs in sorted(byRoutingSite):
-        # "All map to one service in Boise
-        acSites = set(row[1] for row in byRoutingSite[rs])
-        remoteServices = set(row[-1] for row in byRoutingSite[rs])
-        cols = [":Service", ":Acquisition Site", ":Specialty", ":Procedure"]
-        if len(remoteServices) > 1:
-            cols.append(":Remote Service")
-        tbl = MarkdownTable(cols)
-        for row in sorted(byRoutingSite[rs], key=lambda x: x[1]): # group by ac site
-            if len(remoteServices) == 1:
-                row.pop()
-            tbl.addRow(row)
-        if len(acSites) > 1:
-            mu += "__{}__ receives <span class='countHigh'>{:,}</span> types of consult service from <span class='countHigh'>{:,}</span> acquisition sites{} ...\n\n".format(
-                rs, 
-                len(byRoutingSite[rs]), 
-                len(acSites),
-                ", all sent to one remote service __{}__".format(list(remoteServices)[0]) if len(remoteServices) == 1 else "" 
-            )
-        else:
-            mu += "__{}__ has <span class='countHigh'>{:,}</span> services ...\n\n".format(rs, len(byRoutingSite[rs]))
-        mu += tbl.md() + "\n\n"
-    if len(locals):
-        tbl = MarkdownTable([
-            ":Service",
-            ":Acquisition Site",
-            ":Specialty",
-            ":Procedure"
-        ])
-        mu += "The <span class='countHigh'>{:,}</span> locally read services are ...\n\n".format(len(locals))
-        for row in sorted(locals, key=lambda x: x[0]):
-            tbl.addRow(row)
-        mu += tbl.md() + "\n\n"
-    
-    return mu
-
-def webReportConfigurationAcquisitionOld(stationNo):
-
-    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
-    _123_5sOf2006_5841s = set()    
-    resourceIter = FilteredResultIterator(dataLocn, "2006_5841")
-    bySpecialtyIndex = defaultdict(list)
-    for i, resource in enumerate(resourceIter, 1):
-        flattenPropValues(resource)
-        bySpecialtyIndex[resource["specialty_index"]].append(resource)
-        _123_5IEN = re.search(r'123_5\-(\d+)', resource["name"]).group(1)
-        _123_5sOf2006_5841s.add(_123_5IEN)
-
-    # TO ADD: 2005_84 total ie/ how many of the specialties are involved?
-    # TO ELABORATE -- TIU defn ++ KEY CONCEPT specializing local services for IFC,
-    # specifically acquisition ... add direction IN COMBINATION with IFC specialization
-    # of the generic service class 123.5 ... 
-    # <------------ may combine 123.5 with this for one definition from/to, specialty,
-    # document, trigger table
-    def mu2006_5841(bySpecialtyIndex):
-        fmu = """_TELEREADER ACQUISITION SERVICE [2006.5841]_ is the key configuration file for Telereader Image __Acquisition__. All VistA Consults go to a _\"service\"_, a combination of institution that will perform a consult and the specialty required for the consult. This file [1] isolates Telereader acquisition services (_\"Telereader Acquisition Services\"_) and refines them for telereading. It [2] redefines the specialty of a service in nationally recognized terms (ex/ _Dentistry_), [3] specifies which of the institutions of a VistA a service applies to, [4] specifies the title of the TIU created when a \"consult has been read and resulted\".
-
-Its <span class='countHigh'>{:,}</span> entries belong to <span class='countHigh'>{:,}</span> specialties.
-
-""".format(i, len(bySpecialtyIndex))
-        # dropping ("procedure", "Procedure")
-        propMap = [("name", "Service", "DROP"), ("acquisition_site", "Acquisition Site", "IEN"), ("procedure_index", "Procedure Index", "DROP"), ("unread_list_creation_trigger", "Creation Trigger"), ("tiu_note_file", "TIU Note")]
-        for si in sorted(bySpecialtyIndex):
-            suppress=["specialty_index", "procedure"]
-            """
-            KEEP it as suppression is working for col but not row entry!
-            if len(set(resource["acquisition_site"] for resource in bySpecialtyIndex[si])) != len(bySpecialtyIndex[si]):
-                suppress.append("acquisition_site")
-            """
-            tbl = MarkdownTable([":{}".format(pmap[1]) for pmap in propMap if pmap[0] not in suppress])
-            for resource in bySpecialtyIndex[si]:            
-                tbl.addRow(rowUp(resource, propMap, True, suppress=suppress))
-            fmu += "_{}_ has <span class='countHigh'>{:,}</span> ...\n\n".format(si.split(" [")[0], len(bySpecialtyIndex[si]))
-            fmu += tbl.md() + "\n\n"
-            
-        fmu += "Note that _DICOM HEALTHCARE PROVIDER SERVICE [2006.5831]_ which configures the _DICOM Gateway_ also gets entries for the Telereader Acquisition Services.\n\n"
-        return fmu
-       
-    """
-    Probably not enough ie/ only PLACER services and not fillers! Filler Consults
-    have more and different ones (what sets them up?)
-    
-    TODO: why ifcServiceByIEN / reduce123_5ForIFC and then this again?
-    ... issue that some not IFC? ... REDO and combine with 123_5 walk used
-    by ac ie/ don't walk twice!
-    """     
-    def mu123_5(dataLocn, _123_5sOf2006_5841s):
-        propMap = [
-                ("service_name", "Service", "DROP"),
-                ("ifc_routing_site", "Remote Reading Site"),
-                ("ifc_remote_name", "Remote Name"),
-                ("associated_stop_code", "Stop Code(s)")
-                # ("service_usage", "Status")
-        ]    
-        resourceIter = FilteredResultIterator(dataLocn, "123_5")
-        tbl = MarkdownTable([":{}".format(pmap[1]) for pmap in propMap])
-        for i, resource in enumerate(resourceIter, 1):
-            ien = resource["_id"].split("-")[1]
-            if ien not in _123_5sOf2006_5841s:
-                continue
-            flattenPropValues(resource)
-            if "ifc_routing_site" in resource:
-                resource["ifc_routing_site"] = re.sub(r'( VA MEDICAL CENTER| VAMC)', '', re.sub(r'4\-', '', resource["ifc_routing_site"])) # just want [SNO] and VAMC nix for easy read
-            if "associated_stop_code" in resource:
-                resource["associated_stop_code"] = ", ".join(sorted(list(set(sr["associated_stop_code"] for sr in resource["associated_stop_code"]))))
-            tbl.addRow(rowUp(resource, propMap, False))
-        # And the property remote_consult_file entry maps the id at the other site ie/
-        # filler back to placer and placer over to filler.
-        fmu = """_REQUEST SERVICES [123.5]_ holds this VistA's <span class='countHigh'>{:,}</span> consult services. Only <span class='countHigh'>{}</span> are referenced from _TELEREADER ACQUISITION SERVICE [2006.5841]_. Those with a _Remote Reading Site_ involve reading in a remote VistA. Those without involve local reading within this VistA. A (placer) consult created with the _Service_ name results in a (filler) consult with the service name _Remote Name_ in the _Remote Reading Site_. 
-
-""".format(i, len(_123_5sOf2006_5841s))
-        fmu += tbl.md() + "\n\n"
-        return fmu
-        
-    mu = "### Acquisition Role\n\n"
-    mu += mu2006_5841(bySpecialtyIndex)
-    mu += mu123_5(dataLocn, _123_5sOf2006_5841s)
-    
+def webReportReading(stationNo, stationName, onAndAfterDay, upToDay):
+    mu = "## Reading\n\n" 
+    mu += webReportReadingConfiguration(stationNo, stationName)
+    # mu += webReportReadingActivity(stationNo, onAndAfterDay, upToDay)
     return mu
     
-def webReportConfigurationReader(stationNo):
+def webReportReadingConfiguration(stationNo, stationName):
         
     # > The TELEREADER ACQUISITION SITE file (#2006.5842) contains the list of the
     # Acquisition Sites for each Reading Site. This list must also include the Reading     
@@ -600,12 +541,13 @@ def webReportConfigurationReader(stationNo):
         tbl = MarkdownTable([":{}".format(pmap[1]) for pmap in propMap])
         includesItselfMU = ""
         for i, resource in enumerate(sorted(resourceIter, key=lambda x: x["primary_site"]["label"], reverse=True), 1):
+            resource["name"]["label"] = re.sub(r'( VAMC| CBOC| VA CLINIC| VA MEDICAL CENTER)', "", resource["name"]["label"])
             flattenPropValues(resource)
             if re.search(stationNo, resource["primary_site"]):
                 includesItselfMU = ", including itself"
             tbl.addRow(rowUp(resource, propMap, True, suppress=["status"]))
 
-        fmu = """Despite the name, _TELEREADER ACQUISITION SITE [2006.5842]_ configures a VistA for __Reading__, not acquisition. It lists __the sites a VistA can read from__. This list will include the Reading Site itself if it is to read images acquired locally. In this VistA, it has <span class='countHigh'>{:,}</span> entries{} ... 
+        fmu = """Despite the name, _TELEREADER ACQUISITION SITE [2006.5842]_ configures a VistA for __Reading__, not acquisition. It lists __the sites a VistA can read from__. This list will include the Reading Site itself if it reads images acquired locally. In this VistA, it has <span class='countHigh'>{:,}</span> entries{} ... 
 
 """.format(i, includesItselfMU)
         fmu += tbl.md() + "\n\n"
@@ -631,12 +573,42 @@ def webReportConfigurationReader(stationNo):
         
     dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
 
-    mu = "### Reader Role\n\n"
+    mu = "### Configuration\n\n"
     mu += mu2006_5842(dataLocn)
     mu += mu2006_5843(dataLocn)
     
     return mu
     
+"""
+ifc_role (and how to know filler and get types -- 531)
+
+ordering_facility remote?
+to_service (Can get to?)
+tiu_result_narrative (as done locally?) or?
+
+Walk and see if F:FILLER + ifc_remote_service_name + routing_facility (ex/ SPO etc)
+"""
+def webReportReadingActivity(stationNo, onAndAfterDay="", upToDay=""):
+
+    type123YR1, sts123YR1 = splitTypeDatas(stationNo, "123", reductionLabel="YR1")
+    sts123YR1C = combineSubTypes(sts123YR1, ["routing_facility", "to_service"])
+    rf = Counter()
+    # to_service, routing_facility
+    for st in sts123YR1C:
+        if "ordering_facility" in st and "ifc_role" in st and singleValue(st, "ifc_role") == "F:FILLER": 
+            rf[singleValue(st, "routing_facility")] += 1
+            if not re.search(r'TELE-EYE', singleValue(st, "to_service")):
+                continue
+            # print(json.dumps(st, indent=4))
+            print(st["_total"])
+            print("\tTo Service", singleValue(st, "to_service"))
+            print("\tOrdering Facility", singleValue(st, "ordering_facility"))
+            print("\tRouting Facility", singleValue(st, "routing_facility")) # expect equal ordering or is this the main VistA?
+            # comes into 1416 == TELE-EYE IMAGING BOISE (will be > 1)
+    print(rf)
+        
+# ################################# Utils #############################
+    
 def rowUp(resource, propMap, enforceMapAll=True, suppress=[]):
     propsDontSee = ["type", "_id", "label"]
     propsDontSee.extend(suppress)
@@ -819,7 +791,7 @@ def main():
         stationNo = sys.argv[1]
     except IndexError:
         raise SystemExit("Usage _EXE_ STATIONNO")
-
+    
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     if not os.path.isdir(userSiteDir):
         raise SystemExit("Expect User Site to already exist with its basic contents")
