@@ -16,6 +16,10 @@ from .userClassifier import UserClassifier
 from .USER_CONSTANTS import SSNO_DEFINITIONS
 
 """
+FIRST TODO:
+  * ANC/Alaska and no POSTMASTER?
+  * HL7 patterns in Classifier? ie/ correlate the two ... 3.081 + the HL7 log entry on SPQ?
+
 Basic User Type Summary/Overview
 
 Unlike the stopcode centric reports (off locations), these are used and institution centered. Approach is PER USER STs and then Categorize Users (=> STs) as REMOTE
@@ -28,6 +32,9 @@ Requires:
 
 TODO: classification is work in progress - remotes from non key institution, locals
 from non subordinate divisions etc.
+
+Graph It To Emulate:
+https://matplotlib.org/gallery/showcase/bachelors_degrees_by_gender.html#sphx-glr-gallery-showcase-bachelors-degrees-by-gender-py - any dimension for users over time ... types of user?
 """
 def webReportUser(stationNo, deidentify=False):
 
@@ -297,7 +304,7 @@ There are <span class='yellowIt'>{:,}</span> active __Proxy Machine Users__ (use
                 userInfo["date_entered"] if "date_entered" in userInfo else "", 
                 reportAbsAndPercent(st["_total"], totalProxySignOnCount),
                 muSignOnPeriod(st),
-                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5) 
+                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5) if "ipv4_address" in st else "&nbsp;"
             ]
         else:           
             row = [
@@ -308,7 +315,7 @@ There are <span class='yellowIt'>{:,}</span> active __Proxy Machine Users__ (use
                 keysMU,
                 reportAbsAndPercent(st["_total"], totalProxySignOnCount),
                 muSignOnPeriod(st),
-                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
+                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5) if "ipv4_address" in st else "&nbsp;",
                 durMU,
                 unexpectedMU
             ]
