@@ -5,6 +5,8 @@ import sys
 import json
 import re
 from collections import defaultdict, Counter
+from datetime import datetime
+from dateutil.relativedelta import relativedelta
 
 import matplotlib.pyplot as plt
 import matplotlib.ticker as tick
@@ -112,13 +114,14 @@ class Visualize:
         
         Ex/ 
         
-            data = [
+            data = [ (aka rows)
                 ("val col1", "val col2", "val col3") # of row 1
                 ("val col1", "val col2", "val col3") # of row 2
             ]
             columns: ["col1 name", "col2 name", "col3 name"],
-            rows: ["row 1", "row 2"]
+            rows: ["row1 name", "row2 name"] (aka rowLabels)
         
+        Cheat Sheet: https://towardsdatascience.com/data-wrangling-in-pandas-a-downloadable-cheatsheet-84326d255a7b
         """
         df = pd.DataFrame(
             data, 
@@ -148,23 +151,23 @@ class Visualize:
         plotFilePrefix = f'{plotName}CategoryBSV'    
         return self.__flushPlots(plotFilePrefix)
                         
-    def plotDailies(self, dailies, title, plotName, vistaNamesPlus, start='2020-01', end='2020-02'):
-        plotDailies(dailies, title, vistaNamesPlus, start, end)
+    def plotDailies(self, dailies, title, plotName, colNameById, start='2020-01', end='2020-02'):
+        plotDailies(dailies, title, colNameById, start, end)
         plotFilePrefix = f'{plotName}Dailies{start}-{end}' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
-    def plot365RollingMean(self, dailies, title, plotName, vistaNamesPlus, start="1987-01", end=""):
-        plot365RollingMean(dailies, title, vistaNamesPlus, start, end)
+    def plot365RollingMean(self, dailies, title, plotName, colNameById, start="1987-01", end=""):
+        plot365RollingMean(dailies, title, colNameById, start, end)
         plotFilePrefix = f'{plotName}365RollingMean{start}-{end}' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
-    def plotMonthlyResamples(self, dailies, title, plotName, vistaNamesPlus, start='2018-01', end='2020-02'):
-        plotMonthlyResamples(dailies, title, vistaNamesPlus, start, end)
+    def plotMonthlyResamples(self, dailies, title, plotName, colNameById, start='2018-01', end='2020-02'):
+        plotMonthlyResamples(dailies, title, colNameById, start, end)
         plotFilePrefix = f'{plotName}MonthlyResamples{start}-{end}' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
-    def plotTotalsPerYear(self, dailies, title, plotName, vistaNamesPlus, start="2015", end="2020"):
-        plotTotalsPerYear(dailies, title, vistaNamesPlus, start, end)
+    def plotTotalsPerYear(self, dailies, title, plotName, colNameById, start="2015", end="2020", snosOnly=True):
+        plotTotalsPerYear(dailies, title, colNameById, start, end, snosOnly)
         plotFilePrefix = f'{plotName}TotalsPerYear{start}-{end}' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
@@ -218,21 +221,30 @@ def plotIsHasBH(countsNTotal, usePerc=True):
     - [2] side by side comparison of percent of one attribute (ex/ Male)
     for many entities (ex/ VistA 1, VistA 2 ...). Entities are the Y Axis and the 
     LVALUE of the dictionary passed in.
+    but not for category/enum information (ex/ Male vs Female, ALL vs Some VistAs)
     
     Options: usePerc - use percentages vs usePerc = False => absolutes shown
-    
-    Not For: category/enum information (ex/ Male-Female, ALL | 1 | 2 | 3 VistAs etc)
-    
+        
         {
             "__entityName": ...
             "__total": #,
-            "Y1": # <= total,
+            "assertion 1": # <= total, (ex/ in one vista, sent to other sites ...)
             ...
         }
         
     The Percents are calculated inside this function. The #'s passed are absolutes. The
     attributes are reordered by size.
     
+    Note: alternative often used is a DL with form
+    
+        blah blah ..
+                  <---- newline
+            assertion 1
+             : %  <---- one space, colon
+                  <---- newline til next entry
+            assertion 2
+             : %
+    
     TODO (cosmetic)
     - see if can do a DF perc (see tip at bottom of this: 
     https://python-graph-gallery.com/13-percent-stacked-barplot/ with axis etc)
@@ -290,20 +302,25 @@ def plotIsHasBH(countsNTotal, usePerc=True):
     
 def plotCategoryBH(df, title):
     """
-    Category (vs isHas) Bar Horizontal for:
-    - different items with the same category breakdowns 
-    ... do more to refine (could do all same totals => show total?)
+    Category (vs isHas) Bar Horizontal for different items with the same 
+    category breakdowns. Alternative to pie charts.
     
-    Tip: put a # based message in the title
-        
+                       title (tip: put a # message in the title)
+    
+    rowLabel1    |    %col2         | %col2     | %col3 | #
+    
+    rowLabel2    |
+   
+    where key will be col1, col2, col3 and the rows have data with the same breakdown (col1, ...) but can be different orders ie/ total up differently. Ex/ records and files.
+           
     Accepts df. Ex/
-        rows = ["consults", "services", "users", "patients"]
-        columns = ["non ifc", "ifc", "both"]
-        data = [(89361, 5907, 0), (262, 209, 0), (390, 336, 44), (13151, 3454, 500)]    
+        rowLabels = ["rowLabel1", "rowLabel2", "rowLabel3", "rowLabel4"]
+        cols = ["col1", "col2", "col3"]
+        rows = [(89361, 5907, 0), (262, 209, 0), (390, 336, 44), (13151, 3454, 500)]    
         df = pd.DataFrame(
-            data,
-            columns=columns,
-            index=rows
+            rows,
+            columns=cols,
+            index=rowLabels
         )
     Will turn into percentages.
     (Tip: df = df.drop("services"))
@@ -313,7 +330,8 @@ def plotCategoryBH(df, title):
     - consider % on right if #'s don't all total to the same ie/ ala my % of % tables
     as this is a replacement and also order down on those? ie/ lessor total % in lower
     rows
-    https://matplotlib.org/3.1.1/gallery/lines_bars_and_markers/horizontal_barchart_distribution.html#sphx-glr-gallery-lines-bars-and-markers-horizontal-barchart-distribution-py <--- NOT this method but the look side to side is ok
+    
+    REF:    https://matplotlib.org/3.1.1/gallery/lines_bars_and_markers/horizontal_barchart_distribution.html#sphx-glr-gallery-lines-bars-and-markers-horizontal-barchart-distribution-py <--- NOT this method but the look side to side is ok
     
     """        
 
@@ -478,7 +496,12 @@ def plotCategoryBSV(totalsByCatagByStackId, title, usePerc=False): # bar, stacke
     width = .75 # ie/ leave .125 each side between bars
         
     bars = []
-    xs = [-0.15, 0.85, 1.85, 2.85, 3.85, 4.85, 5.85]
+    # xs = [-0.15, 0.85, 1.85, 2.85, 3.85, 4.85, 5.85, 6.85]
+    xs = [-0.15]
+    nxt = 0.85
+    for j in range(1, len(totalsByCatagByStackId)):
+        xs.append(nxt)
+        nxt += 1
     ttlsSoFar = [0 for i in range(0, len(totalsByCatagByStackId))]
     firstTTLs = None
     def formatNumber(fnum):
@@ -634,6 +657,16 @@ Based on https://www.dataquest.io/blog/tutorial-time-series-analysis-with-pandas
 - matplotlib
 - sns (need more info)
 
+Consider weekday out -- 
+
+    from pandas.tseries.offsets import BDay  
+    isBusinessDay = BDay().onOffset
+    csv_path = 'C:\\Python27\\Lib\\site-packages\\bokeh\\sampledata\\daylight_warsaw_2013.csv'
+    dates_df = pd.read_csv(csv_path)
+    match_series = pd.to_datetime(dates_df['Date']).map(isBusinessDay)
+    dates_df[match_series]
+    Crude df[df.index.dayofweek < 5]
+
 TODO: Typer and More
 - Fill in gaps in Patient date created data
 - Feed to subtyping (needs to be more compact if its to keep dailies!)
@@ -657,8 +690,10 @@ sns' boxplots as no great monthly variation to showcase ...
         ax.set_xlabel('')
     plt.show()
     return
+    
+# TODO: allow end="" ie/ start:"" works or?
 """
-def plotDailies(dailies, title, vistaNamesPlus, start='2020-01', end='2020-02'):
+def plotDailies(dailies, title, colNameById, start='2020-01', end='2020-02'):
 
     # Use seaborn style defaults, bg etc
     sns.set() 
@@ -667,10 +702,16 @@ def plotDailies(dailies, title, vistaNamesPlus, start='2020-01', end='2020-02'):
     # Hone in further and add a marker and line style
     # ... must plot directly with matplotlib and not the facade as date
     # calculation for the date formatters differs a little and sets markers off
-    for sno in vistaNamesPlus: # not 'columns' as > sno's in cols now
-        if not re.match(r'\d+$', sno):
+    for colId in colNameById: # not 'columns' as > colId's in cols now
+        """
+        FIX ABOVE for user dir etc -- TODO not here
+        
+        if not re.match(r'\d+$', colId):
             continue # exclude ALL but VistAs ... no 'overall' or total users
-        ax.plot(dailies.loc[start:end, sno], marker='o', linestyle='-', label=vistaNamesPlus[sno])
+        """
+        dailiesLocCol = dailies.loc[start:end, colId]
+        legendLabel = f'{colNameById[colId]} {int(dailiesLocCol.min())}/{int(dailiesLocCol.median())}/{int(dailiesLocCol.max())}'
+        ax.plot(dailiesLocCol, marker='o', linestyle='-', label=legendLabel)
     ax.set_title(title)
     ax.set_ylabel('Daily Addition')
     ax.xaxis.set_major_locator(mdates.WeekdayLocator(byweekday=mdates.MO))
@@ -678,7 +719,7 @@ def plotDailies(dailies, title, vistaNamesPlus, start='2020-01', end='2020-02'):
     ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%d')) # '%b %d'
     # ax.xaxis.grid(color='blue', alpha=0.5) -- work the axis color for 
     ax.legend()
-    
+        
     plt.tight_layout()
     
 """
@@ -688,16 +729,20 @@ def plotDailies(dailies, title, vistaNamesPlus, start='2020-01', end='2020-02'):
 ...
 > Rolling window operations are another important transformation for time series data. Similar to downsampling, rolling windows split the data into time windows and and the data in each window is aggregated with a function such as mean(), median(), sum() ... unlike downsampling, where the time bins do not overlap and the output is at a lower frequency than the input, rolling windows overlap and “roll” along at the same frequency as the data, so the transformed time series is at the same frequency as the original time series.
 """
-def plot365RollingMean(dailies, title, vistaNamesPlus, start="1987-01", end=""):
+def plot365RollingMean(dailies, title, colNameById, start="1987-01", end="2021-12"):
         
     # 365-day rolling mean 
-    _365DayRollingMeans = dailies.loc[start:, list(vistaNamesPlus)].rolling(window=365, center=True).mean()    
+    _365DayRollingMeans = dailies.loc[start:end, list(colNameById)].rolling(window=365, center=True).mean()    
     sns.set() # makes bg and lines
     fig, ax = plt.subplots(figsize=(11, 6))
-    for sno in vistaNamesPlus:
+    for colId in colNameById:
+        """
+        TODO: fix above for users etc
+        
         if not re.match(r'\d+$', sno):
             continue # exclude ALL but VistAs ... no 'overall' or total users
-        ax.plot(_365DayRollingMeans[sno], label=vistaNamesPlus[sno])
+        """
+        ax.plot(_365DayRollingMeans[colId], label=colNameById[colId])
     # Set x-ticks to yearly interval, adjust y-axis limits, add legend and labels
     ax.xaxis.set_major_locator(mdates.YearLocator(2))
     ax.xaxis.set_major_formatter(mdates.DateFormatter('%y'))
@@ -713,15 +758,37 @@ resample the data to monthly frequency, aggregating with sum totals instead of t
 Note: article used stacked for some subdata
     opsd_monthly[['Wind', 'Solar']].plot.area(ax=ax, linewidth=0)
 """
-def plotMonthlyResamples(dailies, title, vistaNamesPlus, start='2018-01', end='2020-02'):
+def plotMonthlyResamples(dailies, title, colNameById, start='2018-01', end='2020-02'):
     
-    monthlies = dailies[list(vistaNamesPlus)].resample('M').sum()
+    # TODO: add check to make sure end gives full month to play with from
+    # passed in data (ONLY CHECKING START NOW!)
     
+    # TODO: may change to "end" and "timeBack" ala utility => easier to QA inside
+    # graph and no duplication logic <----- see timeBack in CSV prod util
+    # Must make sure that start is supported by data frame input and prep to resample
+    # a month back from asked for support date to avoid dip while also avoiding need
+    # to resample the whole of a large data collection
+    startDay = start
+    if len(startDay) == 4:
+        startDay = f'{startDay}-01-01'
+    elif len(startDay) == 7:
+        startDay = f'{startDay}-01'  
+    startDayDT = datetime.strptime(startDay, "%Y-%m-%d")
+    startDayLess1MTHDT = startDayDT - relativedelta(months=1)
+    startDayLess1MTH = datetime.strftime(startDayLess1MTHDT, "%Y-%m-%d")
+    firstRowDay = datetime.strftime(dailies.index[0].to_pydatetime(), "%Y-%m-%d")
+    if firstRowDay > startDayLess1MTH:
+        raise Exception(f'Can\'t monthly resample from start date less a month, {startDayLess1MTH}, as first data date, {firstRowDay}, comes after that')
+    # Resample going back a month to make sure won't start on downturn
+    dailies = dailies.loc[startDayLess1MTH:end]
+
+    monthlies = dailies[list(colNameById)].resample('M').sum()
+        
     sns.set() # makes bg and lines
     fig, ax = plt.subplots()
-    for snop in vistaNamesPlus: # snop as showing overall if present too (for User)
+    for colId in colNameById: 
         # color='black', green orange red
-        ax.plot(monthlies.loc[start:end, snop], label=vistaNamesPlus[snop])
+        ax.plot(monthlies.loc[start:end, colId], label=colNameById[colId])
     # ax.xaxis.set_major_locator(mdates.YearLocator())
     # ax.xaxis.set_major_locator(mdates.WeekdayLocator(byweekday=mdates.MO))
     # Format x-tick labels as 3-letter month name and day number (strftime form)
@@ -754,18 +821,21 @@ TODO: add in plt.annotate('Notice something?', xy=('2014-01-01', 30), xytext=('2
 ... consider doing manuals like this on top of returned plots => save is a separate
 step?
 """
-def plotTotalsPerYear(dailies, title, vistaNamesPlus, start="2015", end="2020"):
+def plotTotalsPerYear(dailies, title, colNameById, start="2015", end="2020", snosOnly=True):
+
+    # TODO: FIX THIS SO NO SNOS etc -- just column names
 
     # Make sure only get VistA (sno) columns, not the total user | patient
-    snos = dict((vnp, vistaNamesPlus[vnp]) for vnp in vistaNamesPlus if re.match(r'\d+$', vnp))
+    snos = dict((vnp, colNameById[vnp]) for vnp in colNameById if re.match(r'\d+$', vnp)) if snosOnly else colNameById
     
-    annuals = dailies[list(snos)].resample('A').sum()
+    dailiesRange = dailies.loc[start:end]
+    annuals = dailiesRange[list(snos)].resample('A').sum()
     annuals = annuals.set_index(annuals.index.year)
     annuals.index.name = 'Year'
 
     sns.set() # makes bg and lines    
-    
-    ax = annuals.loc[start:end].plot.bar()
+
+    ax = annuals.plot.bar()
     
     ax.set_title(title)
     
